package core

import "context"

type ILogger interface {
	BLFatal(module string, d ...interface{})
	BLFatalF(module string, format string, d ...interface{})
	BLFatalC(module string, data map[string]string)

	BLPanic(module string, d ...interface{})
	BLPanicF(module string, format string, d ...interface{})
	BLPanicC(module string, data map[string]string)

	BLError(module string, d ...interface{})
	BLErrorF(module string, format string, d ...interface{})
	BLErrorC(module string, data map[string]string)

	BLWarn(module string, d ...interface{})
	BLWarnF(module string, format string, d ...interface{})
	BLWarnC(module string, data map[string]string)

	BLInfo(module string, d ...interface{})
	BLInfoF(module string, format string, d ...interface{})
	BLInfoC(module string, data map[string]string)

	BLVerbose(module string, d ...interface{})
	BLVerboseF(module string, format string, d ...interface{})
	BLVerboseC(module string, data map[string]string)

	BLDebug(module string, d ...interface{})
	BLDebugF(module string, format string, d ...interface{})
	BLDebugC(module string, data map[string]string)

	GetModuleLogger(module string) IModuleLogger
	DiscardSubLoggerByPrefix(prefix string)
}

type IModuleLogger interface {
	Fatal(d ...interface{})
	FatalF(format string, d ...interface{})
	FatalC(data map[string]string)

	Panic(d ...interface{})
	PanicF(format string, d ...interface{})
	PanicC(data map[string]string)

	Error(d ...interface{})
	ErrorF(format string, d ...interface{})
	ErrorC(data map[string]string)

	Warn(d ...interface{})
	WarnF(format string, d ...interface{})
	WarnC(data map[string]string)

	Info(d ...interface{})
	InfoF(format string, d ...interface{})
	InfoC(data map[string]string)

	Verbose(d ...interface{})
	VerboseF(format string, d ...interface{})
	VerboseC(data map[string]string)

	Debug(d ...interface{})
	DebugF(format string, d ...interface{})
	DebugC(data map[string]string)

	GetModuleName() string
	GetSubLog(suffix string) IModuleLogger

	CloseSubLog(suffix string)
	CloseThisLog()
}

type IModuleLogLevelLimiter interface {
	// LimitLogLevel limit log level for this module logger
	// level parameter:
	//   0 - no limit
	//   1 - under DEBUG (VERBOSE, INFO, WARN, ERROR, PANIC, FATAL) will be emit
	//   2 - under VERBOSE (INFO, WARN, ERROR, PANIC, FATAL) will be emit
	//   3 - under INFO (WARN, ERROR, PANIC, FATAL) will be emit
	//   4 - under WARN (ERROR, PANIC, FATAL) will be emit
	//   other value will performance as 0
	LimitLogLevel(level uint8)
}

type IAppFramework interface {
	ILogger
	GetContext() context.Context
	IsGlobalDebugMode() bool
	SetGlobalDebugMode(en bool)
}
